/***********************************************************************
*
*      Projet  : DilibSxml
*      Module  : SxmlProc
*      Fichier : Source/SxmlDomProc.c
*      Auteur  : J. Ducloy
*
*      Derniere mise a jour Decembre 2000
*      $Id: SxmlDomProc.c,v 1.4 2003/07/10 07:33:18 parmentf Exp $
*
************************************************************************
* 
*     Copyright (C) 2001 CNRS INIST
*
************************************************************************/
/**
   @file

   @brief All functions included in the dom: namespace

   This file implements functions included in the "dom:" namespace:
   - dom:print
   - dom:and
   - dom:firstChild
   - dom:lastChild
   - dom:setVar
   - dom:getVar
   - dom:inputRecord
   
   @since Dec 2000
   @author &copy; 2000 INIST-CNRS
   @author Jacques DUCLOY
 */

#include "SxmlDomProc.h"

/**
   Prints on stdout all the evaluations of the children of @a x1 in
   the context of @a p1.

   @param p1
   @param x1  node to evaluate and print
   @return the last node printed.
 */
SxmlNode *SxmlDomProcPrint(SxmlProcessor *p1,
			 SxmlNode      *x1)
{
  SxmlNode *x2;
  SxmlNode *x3;

  SxmlReset(x1);
  while((x2=SxmlNextNode(x1)))
    {
      if((x3=SxmlProcessorEvalToNode(p1,x2)))SxmlPrint(x3);
    }
  return (x3);
}

SxmlNode *SxmlDomProcPrintNewLine(SxmlProcessor *p1,
			 SxmlNode      *x1)
{
  SxmlNode *x2;
  SxmlNode *x3;

  SxmlReset(x1);
  while((x2=SxmlNextNode(x1)))
    {
      if((x3=SxmlProcessorEvalToNode(p1,x2)))
	{
	  SxmlPrint(x3);
	  putchar('\n');
	}
    }
  return (x3);
}

/**
   Evaluates all children of @a x1 in the context of @a p1. If any of
   them is NULL, returns NULL, else returns the last one.
   
   @param p1
   @param x1 node to test
   @return the last evaluated node
 */
SxmlNode *SxmlDomProcAnd(SxmlProcessor *p1,
		       SxmlNode      *x1)
{
  SxmlNode *x2;
  SxmlNode *x3;
  
  SxmlReset(x1);
  while((x2=SxmlNextNode(x1)))
    {
      x3=SxmlProcessorEvalToNode(p1,x2);
      if(!x3)return NULL;
    }
  return (x3);
}

SxmlNode *SxmlDomProcOr(SxmlProcessor *p1,
		       SxmlNode      *x1)
{
  SxmlNode *x2;
  SxmlNode *x3;
  
  SxmlReset(x1);
  while((x2=SxmlNextNode(x1)))
    {
      x3=SxmlProcessorEvalToNode(p1,x2);
      if(x3)return x3;
    }
  return NULL;
}



/**
   Evaluates the first parameter of @a x1 and returns the first child
   of the evaluation.
   
   @param p1
   @param x1
 */
SxmlNode *SxmlDomProcFirstChild(SxmlProcessor *p1,
			      SxmlNode      *x1)
{
  SxmlNode *x2;
  if ((x2=SxmlProcessorEvalFirstParam(p1,x1)))
    return SxmlFirstChild(x2);
  else 
    return NULL;
}

/**
   Evaluates the first parameter of @a x1 and returns the last child
   of the evaluation.

   @param p1
   @param x1
 */
SxmlNode *SxmlDomProcLastChild(SxmlProcessor *p1,
			     SxmlNode      *x1)
{
  SxmlNode *x2;
  if ((x2=SxmlProcessorEvalFirstParam(p1,x1)))
    return SxmlLastChild(x2);
  else 
    return NULL;
}

/**
   Sets a variable in @a p1's @ref SxmlMemory, with @a x1's "name"
   attribute, to the evaluation of @a x1's first child in @a p1's
   context.

   @param p1
   @param x1
   @returns the value set to the variable
 */
SxmlNode *SxmlDomProcSetVar(SxmlProcessor *p1,
			  SxmlNode      *x1)
{
  char      *n1;
  SxmlNode   *x2;
  SxmlNode   *x3;
  SxmlMemory *mem1;
  n1          = SxmlGetAttribute(x1,"name");
  if (!x1) x2 = NULL;
  else     x2 = SxmlProcessorEvalToNode(p1,SxmlFirstChild(x1));
  x3   = SxmlNodePointerCreate(x2);
  mem1 = SxmlProcessorMemory(p1);
  SxmlSearchTableSetKeyItem(SxmlMemorySymbolTable(mem1),n1, (char *)x3);
  return x2;
}

/**
   @param p1
   @param x1
   @returns SxmlStdinRecord
 */
SxmlNode *SxmlDomProcRecord(SxmlProcessor *p1,
			  SxmlNode      *x1)
{
  return(SxmlStdinRecord);
}

/**
   Returns the value of the variable which name is in @a x1's "name" attribute.
   This value is found in the SymbolTable of the @ref SxmlMemory of @a p1.

   @param p1
   @param x1
   @return the value of the variable which name is in @a x1 attributes
 */
SxmlNode *SxmlDomProcGetVar(SxmlProcessor *p1,
			  SxmlNode      *x1)
{
  char      *n1;
  SxmlMemory *mem1;
  SxmlNode   *x3;
  SxmlNode   *x4 = NULL;
  n1   = SxmlGetAttribute(x1,"name");
  mem1 = SxmlProcessorMemory(p1);
  x3   = SxmlSearchTableGetKeyedItem(SxmlMemorySymbolTable(mem1),n1);
  if(x3)
    x4 = SxmlNodePointerValue(x3);
  return (x4);
}

/**
   Adds to @a mem1 several functions:
   - dom:print
   - dom:and
   - dom:firstChild
   - dom:lastChild
   - dom:setVar
   - dom:getVar
   - dom:inputRecord

   @param mem1
   @returns @a mem1
 */
SxmlMemory *SxmlDomMemoryInit(SxmlMemory *mem1)
{
  SxmlMemorySetXmlBuiltin(mem1,"dom:print"         ,SxmlDomProcPrint);
  SxmlMemorySetXmlBuiltin(mem1,"dom:printNewLine"  ,SxmlDomProcPrintNewLine);
  SxmlMemorySetXmlBuiltin(mem1,"dom:and"           ,SxmlDomProcAnd);
  SxmlMemorySetXmlBuiltin(mem1,"dom:firstChild"    ,SxmlDomProcFirstChild);
  SxmlMemorySetXmlBuiltin(mem1,"dom:lastChild"     ,SxmlDomProcLastChild);
  SxmlMemorySetXmlBuiltin(mem1,"dom:setVar"        ,SxmlDomProcSetVar);
  SxmlMemorySetXmlBuiltin(mem1,"dom:getVar"        ,SxmlDomProcGetVar);
  SxmlMemorySetXmlBuiltin(mem1,"dom:inputRecord"   ,SxmlDomProcRecord);
  return mem1;
}
