/***********************************************************************
*
*      Projet  : DilibSxml
*      Module  : SxmlProc
*      Fichier : SxmlMemory.c
*      Auteur  : J. Ducloy
*
*      Derniere mise a jour Decembre 2001
*      $Id: SxmlMemory.c,v 1.4 2003/06/13 16:32:53 parmentf Exp $
*
************************************************************************
* 
*     Copyright (C) 2001 CNRS INIST
*
************************************************************************/
/**
   @file

   @brief All functions concerning SxmlMemory

   @see SxmlMemory

   @since Dec 2001
   @author &copy; INIST-CNRS
   @author Jacques DUCLOY
 */

/*
   @addtogroup SxmlProc
   @{
 */

#include "SxmlMemory.h"

/**
   One seldom needs more than one SxmlMemory. Here is one that could
   be used, and always available. Internally used.   
 */
SxmlMemory *SxmlMemoryImplicit=NULL;

/**
   Creates an SxmlNode representing an @ref SxmlProcessor's memory, that
   is to say:
   - a symbol table (see SxmlSearchTable),
   - a script part,
   - a local area.

   It attaches also some standard functions as builtins:
   - memory:dump	(@ref SxmlMemoryDump)
   - memory:getVar	(@ref SxmlMemoryGetVar)
   - memory:setVar	(@ref SxmlMemoryBuiltinSetVar)

   @return the newly created @ref SxmlMemory
 */
SxmlMemory *SxmlMemoryCreate()
{
  SxmlMemory      *mem1;
  SxmlSearchTable *symTable;
  mem1=SxmlNodeCreate(SXML_NODE_PROCESSOR);
  SxmlSetNodeName(mem1,"#SxmlMemory");
  SxmlSetNodeSubType(mem1,SXML_PROCESSOR_MEMORY);
  SxmlAppendChild(mem1,symTable=SxmlSearchTableCreate("oscxo"));
  SxmlSetNodeName(symTable,"#symbolTable");
  SxmlSearchTableSetKeyItem(symTable,"@nl",(char *)SxmlTextCreate("\n"));
  SxmlAppendChild(mem1,SxmlElementCreate("#scripts"));
  SxmlAppendChild(mem1,SxmlElementCreate("#localArea"));
  SxmlMemorySetXmlBuiltin(mem1,"memory:dump"	,SxmlMemoryDump);
  SxmlMemorySetXmlBuiltin(mem1,"memory:getVar"	,SxmlMemoryBuiltinGetVar);
  SxmlMemorySetXmlBuiltin(mem1,"memory:setVar"	,SxmlMemoryBuiltinSetVar);
  return mem1;
}


/**
   Creates a default static SxmlMemory, if it is not already created
   (its name is @ref SxmlMemoryImplicit).
   @returns @ref SxmlMemoryImplicit
 */
SxmlMemory *SxmlMemoryImplicitCreate()
{
  if(!SxmlMemoryImplicit)
    SxmlMemoryImplicit=SxmlMemoryCreate();
  return SxmlMemoryImplicit;
}


/**
   Adds the function @a f1 as an item of the symbol table of the
   memory @a mem1, associated with the name @a n1, using an
   SxmlContainer.

   @param mem1 SxmlMemory in which to add the "builtin"
   @param n1   Sxml name of the function to add
   @param f1   pointer to the C function (returning an SxmlNode)
   @return the modified @a mem1
 */
SxmlMemory *SxmlMemorySetXmlBuiltin(SxmlMemory *mem1,
				  char      *n1,
				  SxmlNode   *(*f1)())
{
  SxmlSearchTable *symTable;
  SxmlNode        *c1;
  symTable=SxmlMemorySymbolTable(mem1);
  c1=SxmlContainerSxmlFunctionCreate(f1);
  SxmlSearchTableSetKeyItem(symTable,n1,(char *)c1);
  return mem1;
}


/**
   Adds a function in the "#scripts" part of @a mem1, associated to the
   name @a n1. Creates its "#body" part (empty).
   
   @param mem1	SxmlMemory where to add the function
   @param n1	name of the function to add
   @param par1	unused
   @param b1	unused
   @return the SxmlProcessorFunction created
   
 */
SxmlNode *SxmlMemoryFunctionCreate(SxmlMemory *mem1,
				 char      *n1,
				 SxmlNode   *par1,
				 SxmlNode   *b1)
{
  SxmlNode *f1;	       /* SxmlProcessorFunction */
  f1=SxmlNodeCreate(SXML_NODE_PROCESSOR);
  SxmlSetNodeSubType(f1,SXML_PROCESSOR_FUNCTION);
  SxmlSetNodeName(f1,n1);
  SxmlAppendChild(SxmlMemoryScripts(mem1),f1);
  SxmlAppendChild(f1,SxmlElementCreate("#body"));
  return f1;
}


/**
   Returns the SxmlProcessorFunction from @a mem1 which name is @a n1.

   @param mem1	SxmlMemory to get the function from
   @param n1	name of the function to get
 */
SxmlNode *SxmlMemoryGetFunctionByName(SxmlMemory *mem1,
				    char      *n1)
{
  SxmlNode *f1;
  SxmlNode *st1;
  st1=SxmlMemoryScripts(mem1);
  f1=SxmlGetFirstChildByTagName(st1,n1);
  return f1;
}

/**
   Appends the instruction @a x1 to the body of the function which
   name is @a n1 in @a mem1.

   Creates the function @a n1 if needed.

   @param mem1	SxmlMemory where the function @a n1 is
   @param n1	name of the function in which to append something
   @param x1	part to add to the body of @a n1
   @return the SxmlProcessorFunction where @a x1 was appended
 */
SxmlNode *SxmlMemoryAppendToFunction(SxmlMemory *mem1,
				   char      *n1,
				   SxmlNode *x1)
{
  SxmlNode *f1;		/* SxmlProcessorFunction */
  if (!(f1=SxmlMemoryGetFunctionByName(mem1,n1)))
    {
      f1=SxmlMemoryFunctionCreate(mem1,n1,NULL,NULL);
    }
  SxmlMemoryFunctionAppendBody(f1,x1);
  return f1;
}


/**
   Appends the @a n1 instruction to the body of the function @a f1.

   @param f1	SxmlProcessorFunction from an SxmlMemory
   @param n1	instruction to add to the body of @a f1
   @return the modified @a f1
 */
SxmlNode *SxmlMemoryFunctionAppendBody(SxmlNode *f1,
				     SxmlNode *n1)
{
  SxmlNode *bodyNode;
  bodyNode=SxmlFirstChild(f1);
  SxmlAppendChild(bodyNode,n1);
  return f1;
}

/**
   Adds a variable into the symbol table of @a p1, with the "name"
   attribute of @a x1 and as a value, the first child of @a x1.

   If the variable already exists, its value is changed.

   @param p1	SxmlProcessor
   @param x1	the variable to set
   @return the value set
   @internal @ref SxmlDomProcSetVar seems to do the same. Check.
 */
SxmlNode *SxmlMemoryBuiltinSetVar(SxmlProcessor *p1,
			 SxmlNode      *x1)
{
  char *n1;
  SxmlNode *x2;
  SxmlNode *x3;
  SxmlNode *mem1;
  n1=SxmlGetAttribute(x1,"name");
  if (!x1)x2=NULL;
  else x2=SxmlFirstChild(x1);
  x3=SxmlNodePointerCreate(x2);
  mem1=SxmlProcessorMemory(p1);
  SxmlSearchTableSetKeyItem(SxmlMemorySymbolTable(mem1),n1, (char *)x3);
  return x2;
}

SxmlNode *SxmlMemorySetVar(SxmlMemory *mem1,
			   char     *name,
			   SxmlNode *x1)
{
  SxmlNode *x2;
  x2=SxmlNodePointerCreate(x1);
  SxmlSearchTableSetKeyItem(SxmlMemorySymbolTable(mem1),name,(char *)x2);
  return x2;
}


/**
   Gets the value of of the variable which name is given in the "name"
   attribute of @a x1.

   @param p1	SxmlProcessor
   @param x1	SxmlNode which "#name" attribute is the name of the variable
   @returns a pointer to the SxmlNode that is the value of the variable
   which name is given in the "name" attribute of @a x1.
   @internal @ref SxmlDomProcGetVar seems to do the same. Check.
 */

SxmlNode *SxmlMemoryGetVar(SxmlMemory *mem1,  char *name)
{
  SxmlNode *x3;
  SxmlNode *x4;
  x3=SxmlSearchTableGetKeyedItem(SxmlMemorySymbolTable(mem1),name);
  x4=SxmlNodePointerValue(x3);
  return (x4);
}


SxmlNode *SxmlMemoryBuiltinGetVar(SxmlProcessor *p1,  SxmlNode *x1)
{
  char *n1;
  SxmlNode *x3;
  SxmlNode *x4;
  SxmlMemory *mem1;
  n1=SxmlGetAttribute(x1,"name");
  mem1=SxmlProcessorMemory(p1);
  x3=SxmlSearchTableGetKeyedItem(SxmlMemorySymbolTable(mem1),n1);
  x4=SxmlNodePointerValue(x3);
  return (x4);
}


/**
   Dumps @a p1's SxmlMemory to stdout (Bug trace), and returns it.

   @param p1
   @returns @a p1's @ref SxmlMemory
 */
SxmlMemory *SxmlMemoryDump(SxmlProcessor *p1)
{
  SxmlMemory *mem1;
  mem1=SxmlProcessorMemory(p1);
  SxmlDump(mem1);
  return mem1;
}


/*
   @}
 */
