/***********************************************************************
*
*      Projet  : DilibSxml
*      Module  : SxmlProc
*      Fichier : SxmlProcessor.c
*      Auteur  : J. Ducloy
*
*      Derniere mise a jour Decembre 2001
*      $Id: SxmlProcessor.c,v 1.6 2003/07/29 16:22:12 parmentf Exp $
*
************************************************************************
* 
*     Copyright (C) 2001 CNRS INIST
*
************************************************************************/
/**
   @file

   @brief All functions concerning SxmlProcessor

   @see SxmlProcessor

   @since Dec 2001
   @author &copy; INIST-CNRS
   @author Jacques DUCLOY
 */

#include "SxmlProcessor.h"

/**
   One seldom needs more than one SxmlProcessor. Here is one that could
   be used, and always available. Internally used.
 */
SxmlProcessor  *SxmlProcessorImplicit	= NULL;

/**
   Creates a SxmlProcessor having @a mem1 as SxmlProcessor's memory,
   that is to say: a symbol table, a script part, a local area. It
   attaches also some standard functions as builtins: @c
   "memory:dump".
   
   @param mem1
   @return the created SxmlProcessor
   @see SxmlProcessorDump
*/
SxmlProcessor *SxmlProcessorCreate(SxmlMemory *mem1)
{
  SxmlNode *proc1;
  SxmlNode *ptr1;
  proc1=SxmlNodeCreate(SXML_NODE_PROCESSOR);
  SxmlSetNodeName(proc1,"#processor");
  SxmlSetNodeSubType(proc1,SXML_PROCESSOR);
  SxmlAppendChild(proc1, ptr1=SxmlNodePointerCreate(mem1));
  SxmlSetNodeName(ptr1,"#memory");
  if (!(SxmlGetAttribute(mem1,"init:proc")))
    {
      SxmlSetAttribute(mem1,"init:proc","yes");
      SxmlMemorySetXmlBuiltin(mem1,"proc:dump",SxmlProcessorDump);
    }
  return proc1;
}


/**
   Get the SxmlProcessorImplicit, creates it if it is not yet created.

   @return a pointer to SxmlProcessorImplicit.
 */
SxmlProcessor *SxmlProcessorImplicitCreate()
{
  if(!SxmlProcessorImplicit)
    {
      if(!SxmlMemoryImplicit)SxmlMemoryImplicitCreate();
      SxmlProcessorImplicit
	=SxmlProcessorCreate(SxmlMemoryImplicit);
    }
  return SxmlProcessorImplicit;
}

/**
   If @a x1 is an SxmlElement and its name begins with @c "@", returns
   the SxmlNode containing the attribute value.  If it is an
   SxmlElement, applies the function which name is the one of @a x1, in
   @a proc1, to @a x1.
   
   If the @a x1 is not an SxmlElement returns @a x1.
   
   @param proc1 
   @param x1 
   @return the SxmlNode corresponding to name of @a x1 if it is an
   SxmlElement. Else, return @a x1.
   @see SxmlElement
 */

SxmlNode *SxmlProcessorEvalToNode(SxmlProcessor *proc1,
				SxmlNode *x1)
{
  SxmlMemory *mem1;
  SxmlNode   *symTable;
  SxmlNode   *fNode;
  mem1     = SxmlProcessorMemory(proc1);
  symTable = SxmlMemorySymbolTable(mem1);
  if (!x1)return NULL;
  if (SxmlNodeType(x1)==SXML_NODE_ELEMENT)
    {
      char *nameF1;
      nameF1=SxmlNodeName(x1);
      if(*nameF1=='@')return SxmlSearchTableGetKeyedItem(symTable,nameF1);
      else if((fNode=SxmlSearchTableGetKeyedItem(symTable,nameF1))
	 &&(SxmlNodeSubType(fNode)==SXML_CONTAINER_SXML_FUNCTION))
	{
	  SxmlNode *(*f1)();
	  f1=(SxmlNode *(*)())SxmlNodeValue(fNode);
	  return f1(proc1,x1);
	}
      else return x1;
    }
  else return x1;
}

/**
   If @a x1 has a @c name attribute, get its matching SxmlNode in the
   @ref SxmlMemory of @a proc1.

   Else, evaluate the first child of @a x1 and returns the resulting
   SxmlNode.$

   @param proc1
   @param x1
   @return the first SxmlNode matching the @a x1 in @a proc1
 */
SxmlNode *SxmlProcessorEvalFirstParam(SxmlProcessor *proc1,
				    SxmlNode      *x1)
{  SxmlNode *x2;
  SxmlNode *x3;
  char    *n1;
  if ((n1=SxmlGetAttribute(x1,"name")))
    {
      SxmlMemory *mem1;
      SxmlNode   *x4;
      mem1 = SxmlProcessorMemory(proc1);
      x4   = SxmlSearchTableGetKeyedItem(SxmlMemorySymbolTable(mem1),n1);
      x3   = SxmlNodePointerValue(x4);
    }
  else
    {
      x2 = SxmlFirstChild(x1);
      if (!x2)return NULL;
      x3 = SxmlProcessorEvalToNode(proc1,x2);
    }
  return x3;
}

/**
   Evaluates @a x1, converts it to a string, and returns it.

   @param ctx1 context (with its symbolTable)
   @param x1
   @returns a C string
 */
char *SxmlProcessorEvalToString(SxmlProcessor *ctx1,
			       SxmlNode      *x1)
{
  SxmlNode *x2;
  char    *res = NULL;

  if((x2=SxmlProcessorEvalToNode(ctx1, x1)))
    switch (SxmlNodeType(x2))
      {
      case SXML_NODE_ATTRIBUTE:
	res = SxmlNodeValue(x2);
	break;
      case SXML_NODE_TEXT:
	res = SxmlNodeValue(x2);
	break;
      default:
	res = SxmlToString(x2);
      break;
      }
  return res;
}


/**
   Evaluates @a f1 in the context of @a ctxt1.

   @param ctxt1 (contextual) @ref SxmlProcessor to use
   @param f1	function to evaluate
   @param cNode  
   @return the evaluation of all the parts of the body of @a f1
   @internal cNode seems to be unused
 */
SxmlNode *SxmlProcessorEvalFunction(SxmlProcessor *ctxt1,
				  SxmlNode *f1,
				  SxmlNode *cNode)
{
  SxmlNode *bodyNode;
  SxmlNode *x2;
  SxmlNode *x3;
  bodyNode=SxmlFirstChild(f1);
  SxmlReset(bodyNode);
  while((x2=SxmlNextNode(bodyNode)))
    x3=SxmlProcessorEvalToNode(ctxt1, x2);
  return x3;
}


/**
   Dumps @a p1 on stdout (debug trace).

   @param p1
   @returns @a p1
 */
SxmlProcessor *SxmlProcessorDump(SxmlProcessor *p1)
{
  SxmlDump(p1);
  return p1;
}

